︠df6784cb-311b-44c3-a62b-1c7f3e245543i︠
%md
### Black Scholes formulas for European Call and Put

$$C(S,t)=S N(d_1)-E e^{-r(T-t)}N(d_2),\ \ \  P(S,t)=E e^{-r(T-t)}N(-d_2)-S N(-d_1)$$

where $N(x)=\dfrac{1}{\sqrt(2\pi)}\int_{-\infty}^x e^{\frac{-y^2}{2}}dy = \dfrac{1}{2} \text{ erf}(x) +\dfrac{1}{2}$, $S$ is Asset price, $E$ is Expiry Price,  $T$ is expiry time, $r$ is bank interest rate, $t$ is time,
$d_1=\ln(\dfrac{S}{E})+(r+\dfrac{1}{2}\sigma^2)(T-t))\ \dfrac{1}{\sigma \sqrt{T-t}}$ and $d_2=\ln(\dfrac{S}{E})+(r-\dfrac{1}{2}\sigma^2)(T-t))\ \dfrac{1}{\sigma \sqrt{T-t}}$
︡31bc1885-3ca2-4df5-8741-5b48db660c56︡︡{"done":true,"md":"### Black Scholes formulas for European Call and Put\n\n$$C(S,t)=S N(d_1)-E e^{-r(T-t)}N(d_2),\\ \\ \\  P(S,t)=E e^{-r(T-t)}N(-d_2)-S N(-d_1)$$\n\nwhere $N(x)=\\dfrac{1}{\\sqrt(2\\pi)}\\int_{-\\infty}^x e^{\\frac{-y^2}{2}}dy = \\dfrac{1}{2} \\text{ erf}(x) +\\dfrac{1}{2}$, $S$ is Asset price, $E$ is Expiry Price,  $T$ is expiry time, $r$ is bank interest rate, $t$ is time,\n$d_1=\\ln(\\dfrac{S}{E})+(r+\\dfrac{1}{2}\\sigma^2)(T-t))\\ \\dfrac{1}{\\sigma \\sqrt{T-t}}$ and $d_2=\\ln(\\dfrac{S}{E})+(r-\\dfrac{1}{2}\\sigma^2)(T-t))\\ \\dfrac{1}{\\sigma \\sqrt{T-t}}$"}
︠5a4d77b9-b2b7-4826-9fd6-ea0e4ab89e73︠

︠1f46626b-510f-4a6c-b0bf-563bed296c78︠
S, t, T, E, r, sigma ,y, dt=var('S t T E r sigma y dt')

@interact(layout={'top': [['n', 'm','S0','t0','SM'],['r','sigma','E','T','TM']]})
def showit(n=20,m=20,S0=.8,t0=.5,r=.1,sigma=.2,E=1,T=1,SM=1.5,TM=1):
    def N(x):
    #return 1/sqrt(2*pi)*integral(exp(-y^2/2),(y,-infinity,x))
       return erf(x)/2+1/2
    r=r
    sigma=sigma
    E=E
    T=T
    def C3(S,t):
        d1=(ln(S/E)+(r+1/2*sigma^2)*(T-t))/(sigma*sqrt(T-t))
        d2=(ln(S/E)+(r-1/2*sigma^2)*(T-t))/(sigma*sqrt(T-t))
        return S*N(d1)-E*exp(-r*(T-t))*N(d2)
    Ttab=[(S0.n(digits=4),(TM*(1-i/n)).n(digits=10),C3(S0,TM*(1-i/n)).n(digits=10)) for i in range(n)]
    Stab=[((SM*i/m).n(digits=10),t0.n(digits=4),C3(SM*i/m,t0).n(digits=10)) for i in range(m+1)]
    G=Graphics()
    G+=plot3d(C3,(S,0,SM),(t,0,TM),mesh=false,dots=true,opacity=.5,figsize=10,axes=false)
    G+=line(Ttab,color='red',thickness=1)
    G+=line(Stab,color='green',thickness=1)

    G.show()
    pretty_print(html( '<h3>&nbsp;&nbsp;&nbsp;Graph of the Black Scholes European Call:  x=S Asset, y=t time, z=C Call</h4>'))
    print
    print table([('','Red Line','')]+[('Asset Price','Time','Call Price')]+Ttab)
    print
    print table([('','Green Line','')]+[('Asset Price','Time','Call Price')]+Stab)
︡407849a1-9ba1-4c52-86b1-cb769889bec4︡︡{"interact":{"style":"None","flicker":false,"layout":[[["n",2,null],["m",2,null],["S0",2,null],["t0",2,null],["SM",2,null]],[["r",2,null],["sigma",2,null],["E",2,null],["T",2,null],["TM",2,null]],[["",12,null]]],"id":"07ae9bcb-7e59-43c8-8682-3dc63cad83a5","controls":[{"control_type":"input-box","default":20,"label":"n","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"n","type":null},{"control_type":"input-box","default":20,"label":"m","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"m","type":null},{"control_type":"input-box","default":"0.800000000000000","label":"S0","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"S0","type":null},{"control_type":"input-box","default":"0.500000000000000","label":"t0","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"t0","type":null},{"control_type":"input-box","default":"0.100000000000000","label":"r","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"r","type":null},{"control_type":"input-box","default":"0.200000000000000","label":"sigma","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"sigma","type":null},{"control_type":"input-box","default":1,"label":"E","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"E","type":null},{"control_type":"input-box","default":1,"label":"T","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"T","type":null},{"control_type":"input-box","default":"1.50000000000000","label":"SM","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"SM","type":null},{"control_type":"input-box","default":1,"label":"TM","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"TM","type":null}]},"done":false}︡{"done":true}
︠fbf83825-fb73-4356-9101-65dcd5c0363f︠
var('S t T E r sigma y dt')
r=.1;sigma=.2;E=1;T=1

def N(x):
    #return 1/sqrt(2*pi)*integral(exp(-y^2/2),(y,-infinity,x))
    return erf(x)/2+1/2
def C(S):
    d1=(ln(S/E)+(r+1/2*sigma^2)*(dt))/(sigma*sqrt(dt))
    d2=(ln(S/E)+(r-1/2*sigma^2)*(dt))/(sigma*sqrt(dt))
    return S*N(d1)-E*exp(-r*(dt))*N(d2)
def C3(S,t):
    d1=(ln(S/E)+(r+1/2*sigma^2)*(T-t))/(sigma*sqrt(T-t))
    d2=(ln(S/E)+(r-1/2*sigma^2)*(T-t))/(sigma*sqrt(T-t))
    return S*N(d1)-E*exp(-r*(T-t))*N(d2)
︡75399ccc-2478-4a79-bd49-742ad20781a2︡︡{"stdout":"(S, t, T, E, r, sigma, y, dt)\n","done":false}︡{"done":true}
︠1898be22-d3e9-4883-96bc-73315f16a962︠
pretty_print(('This is $2+2$.'))
︡31e5d316-3085-4af9-ab08-92bcf04c816c︡︡{"html":"<div align='center'>This is $2+2$.</div>","done":false}︡{"done":true}
︠d6b71fae-5cee-4820-b06b-e80a2ecc2cdc︠

︠d1e93f03-7a48-42ad-8dc6-368a83cf11ce︠
help(pretty_print)
︡e337f9af-f76e-4005-bff3-4a1c105ceab6︡︡{"stdout":"Help on function show in module smc_sagews.sage_salvus:\n\nshow(*objs, **kwds)\n    Show a 2d or 3d graphics object (or objects), animation, or matplotlib figure, or show an\n    expression typeset nicely using LaTeX.\n    \n       - display: (default: True); if True, use display math for expression (big and centered).\n    \n       - svg: (default: True); if True, show 2d plots using svg (otherwise use png)\n    \n       - d3: (default: True); if True, show graphs (vertices and edges) using an interactive D3 viewer\n           for the many options for this viewer, type 'import graphics; graphics.graph_to_d3_jsonable?'\n         If false, graphs are converted to plots and displayed as usual.\n    \n       - renderer: (default: 'webgl'); for 3d graphics\n           - 'webgl' (fastest) using hardware accelerated 3d;\n           - 'canvas' (slower) using a 2d canvas, but may work better with transparency;\n           - 'tachyon' -- a ray traced static image.\n    \n       - spin: (default: False); spins 3d plot, with number determining speed (requires mouse over plot)\n    \n       - events: if given, {'click':foo, 'mousemove':bar}; each time the user clicks,\n         the function foo is called with a 2-tuple (x,y) where they clicked.  Similarly\n         for mousemove.  This works for Sage 2d graphics and matplotlib figures.\n    \n    ANIMATIONS:\n    \n       - animations are by default encoded and displayed using an efficiently web-friendly\n         format (currently webm, which is **not supported** by Safari or IE).\n    \n            - ``delay`` - integer (default: 20); delay in hundredths of a\n              second between frames.\n    \n            - gif=False -- if you set gif=True, instead use an animated gif,\n              which is much less efficient, but works on all browsers.\n    \n         You can also use options directly to the animate command, e.g., the figsize option below:\n    \n              a = animate([plot(sin(x + a), (x, 0, 2*pi)) for a in [0, pi/4, .., 2*pi]], figsize=6)\n              show(a, delay=30)\n    \n    \n    EXAMPLES:\n    \n    Some examples:\n    \n        show(2/3)\n        show([1, 4/5, pi^2 + e], 1+pi)\n        show(x^2, display=False)\n        show(e, plot(sin))\n    \n    Here's an example that illustrates creating a clickable image with events::\n    \n        @interact\n        def f0(fun=x*sin(x^2), mousemove='', click='(0,0)'):\n            click = sage_eval(click)\n            g = plot(fun, (x,0,5), zorder=0) + point(click, color='red', pointsize=100, zorder=10)\n            ymax = g.ymax(); ymin = g.ymin()\n            m = fun.derivative(x)(x=click[0])\n            b =  fun(x=click[0]) - m*click[0]\n            g += plot(m*x + b, (click[0]-1,click[0]+1), color='red', zorder=10)\n            def h(p):\n                f0.mousemove = p\n            def c(p):\n                f0(click=p)\n            show(g, events={'click':c, 'mousemove':h}, svg=True, gridlines='major', ymin=ymin, ymax=ymax)\n\n","done":false}︡{"done":true}
︠603cba1e-dd20-4e7d-b2c0-cff38f3a9fe9︠

︠21c52425-8e79-4c8f-a2ca-758b5e52d37c︠

︠054ec4d6-a146-4893-bd1d-9be99c53b071︠

︠b9ec44b9-fbf4-4504-9019-b820e3f51919︠

︠88050a6c-860d-48e7-8d4e-73c10c4f634ai︠
S, t, T, E, r, sigma ,y, t=var('S t T E r sigma y t')
#r=.1;sigma=.2;E=1;T=1


#def C(S):
#    d1=(ln(S/E)+(r+1/2*sigma^2)*(dt))/(sigma*sqrt(dt))
#    d2=(ln(S/E)+(r-1/2*sigma^2)*(dt))/(sigma*sqrt(dt))
#    return S*N(d1)-E*exp(-r*(dt))*N(d2)

@interact(layout={'top': [['n', 'm','SM','TM'],['r','sigma','E','T'],['S0','S1','t0','t1']]})
def showit(n=20,m=20,t0=.5,S0=.8,r=.1,sigma=.2,E=1,T=1,SM=1.5,TM=1,S1=0,t1=0):
    def N(x):
    #return 1/sqrt(2*pi)*integral(exp(-y^2/2),(y,-infinity,x))
        return erf(x)/2+1/2
    r=r
    sigma=sigma
    E=E
    T=T
    def C3(S,t):
        d1=(ln(S/E)+(r+1/2*sigma^2)*(T-t))/(sigma*sqrt(T-t))
        d2=(ln(S/E)+(r-1/2*sigma^2)*(T-t))/(sigma*sqrt(T-t))
        return S*N(d1)-E*exp(-r*(T-t))*N(d2)
    Ttab=[(S0.n(digits=4),(TM*(1-i/n)).n(digits=10),C3(S0,TM*(1-i/n)).n(digits=10)) for i in range(n)]
    if S1>0:
        Ttab1=[(S1.n(digits=4),(TM*(1-i/n)).n(digits=10),C3(S1,TM*(1-i/n)).n(digits=10)) for i in range(n)]
    Stab=[((SM*i/m).n(digits=10),t0.n(digits=4),C3(SM*i/m,t0).n(digits=10)) for i in range(m+1)]
    if t1>0:
        Stab1=[((SM*i/m).n(digits=10),t1.n(digits=4),C3(SM*i/m,t1).n(digits=10)) for i in range(m+1)]
    G=Graphics()
    G+=plot3d(C3,(S,0,SM),(t,0,TM),mesh=false,dots=true,opacity=.5,figsize=10,axes=false)
    G+=line(Ttab,color='red',thickness=1)
    if S1>0:
        G+=line(Ttab1,color='red',thickness=1)
    G+=line(Stab,color='green',thickness=1)
    if t1>0:
        G+=line(Stab1,color='green',thickness=1)

    G.show()
    pretty_print(html( '<h3>&nbsp;&nbsp;&nbsp;Graph of the Black Scholes European Call:  x=S Asset, y=t time, z=C Call</h4>'))
    print
    print table([('','Red Line','')]+[('Asset Price','Time','Call Price')]+Ttab)
    print
    print table([('','Green Line','')]+[('Asset Price','Time','Call Price')]+Stab)
    if S1>0:
        print
        print table([('','Red Line1','')]+[('Asset Price','Time','Call Price')]+Ttab1)
    if t1>0:
        print
        print table([('','Green Line1','')]+[('Asset Price','Time','Call Price')]+Stab1)

︡7dca3440-2f92-4a28-8845-d85458bad08e︡︡{"interact":{"style":"None","flicker":false,"layout":[[["n",3,null],["m",3,null],["SM",3,null],["TM",3,null]],[["r",3,null],["sigma",3,null],["E",3,null],["T",3,null]],[["S0",3,null],["S1",3,null],["t0",3,null],["t1",3,null]],[["",12,null]]],"id":"1a770918-40e4-411d-866b-b0d756aeb6ab","controls":[{"control_type":"input-box","default":20,"label":"n","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"n","type":null},{"control_type":"input-box","default":20,"label":"m","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"m","type":null},{"control_type":"input-box","default":"0.500000000000000","label":"t0","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"t0","type":null},{"control_type":"input-box","default":"0.800000000000000","label":"S0","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"S0","type":null},{"control_type":"input-box","default":"0.100000000000000","label":"r","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"r","type":null},{"control_type":"input-box","default":"0.200000000000000","label":"sigma","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"sigma","type":null},{"control_type":"input-box","default":1,"label":"E","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"E","type":null},{"control_type":"input-box","default":1,"label":"T","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"T","type":null},{"control_type":"input-box","default":"1.50000000000000","label":"SM","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"SM","type":null},{"control_type":"input-box","default":1,"label":"TM","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"TM","type":null},{"control_type":"input-box","default":0,"label":"S1","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"S1","type":null},{"control_type":"input-box","default":0,"label":"t1","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"t1","type":null}]},"done":false}︡{"done":true}
︠8272feed-dcc3-4fa1-bcc0-3aaf1981ea14︠
︡01e75c6f-7408-485d-a90e-02cbdbca31d0︡︡{"done":true}
︠030c0501-7970-48ca-96fe-360f7742431d︠
@interact
def f0(fun=x*sin(x^2), mousemove='', click='(0,0)'):
    click = sage_eval(click)
    g = plot(fun, (x,0,5), zorder=0) + point(click, color='red', pointsize=100, zorder=10)
    ymax = g.ymax(); ymin = g.ymin()
    m = fun.derivative(x)(x=click[0])
    b =  fun(x=click[0]) - m*click[0]
    g += plot(m*x + b, (click[0]-1,click[0]+1), color='red', zorder=10)
    def h(p):
        f0.mousemove = p
    def c(p):
        f0(click=p)
    show(g, events={'click':c, 'mousemove':h}, svg=True, gridlines='major', ymin=ymin, ymax=ymax)

︡a2e0f9a7-4ea5-434d-9dda-efd6610643c6︡︡{"interact":{"style":"None","flicker":false,"layout":[[["fun",12,null]],[["mousemove",12,null]],[["click",12,null]],[["",12,null]]],"id":"c931283d-5c8e-4f77-9a83-86aa14dc5230","controls":[{"control_type":"input-box","default":"x*sin(x^2)","label":"fun","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"fun","type":null},{"control_type":"input-box","default":"","label":"mousemove","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"mousemove","type":"<type 'str'>"},{"control_type":"input-box","default":"(0,0)","label":"click","nrows":1,"width":null,"readonly":false,"submit_button":null,"var":"click","type":"<type 'str'>"}]},"done":false}︡{"done":true}
︠9eec4c91-297f-49f3-abc7-bb946ef333e2︠
︠19096894-e97b-47ea-945f-c9db45734a87︠









